#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import yaml
import logging
import codecs
import os
import sys
from .constant import LOGGER_NAME
from .oss_item import OssItem

_logger = logging.getLogger(LOGGER_NAME)


def parsing_yml(yaml_file, base_path):
    oss_list = []
    license_list = []
    idx = 1
    OLD_YAML_ROOT_ELEMENT = ['Open Source Software Package',
                             'Open Source Package']

    try:
        path_of_yml = os.path.normpath(os.path.dirname(yaml_file))
        base_normpath = os.path.normpath(base_path)
        relative_path = ""
        if path_of_yml != base_normpath:
            relative_path = os.path.relpath(path_of_yml, base_normpath)
        else:
            relative_path = ""
        doc = yaml.safe_load(codecs.open(yaml_file, "r", "utf-8"))
        is_old_format = any(x in doc for x in OLD_YAML_ROOT_ELEMENT)

        for root_element in doc:
            oss_items = doc[root_element]
            if oss_items:
                for oss in oss_items:
                    item = OssItem(relative_path)
                    if not is_old_format:
                        item.name = root_element
                    for key, value in oss.items():
                        if key:
                            key = key.lower().strip()
                        set_value_switch(item, key, value)
                    oss_list.append(item)
                    license_list.extend(item.license)
                    idx += 1
    except yaml.YAMLError:
        _logger.warning(f"Can't parse yaml - skip to parse yaml file: {yaml_file}")
    return oss_list, set(license_list)


def find_all_oss_pkg_files(path_to_find, file_to_find):
    oss_pkg_files = []

    if not os.path.isdir(path_to_find):
        _logger.error(f"Can't find a path: {path_to_find}")
        sys.exit(os.EX_DATAERR)

    try:
        # oss_pkg_files.extend(glob.glob(path_to_find + '/**/'+file, recursive=True)) #glib is too slow
        for p, d, f in os.walk(path_to_find):
            for file in f:
                file_name = file.lower()
                if file_name in file_to_find or (
                        (file_name.endswith("yml") or file_name.endswith("yaml"))
                        and file_name.startswith("oss-pkg-info")):
                    oss_pkg_files.append(os.path.join(p, file))
    except Exception as ex:
        _logger.error(f"Error, find all oss-pkg-info files: {ex}")

    return oss_pkg_files


def set_value_switch(oss, key, value):
    if key in ['oss name', 'name']:
        oss.name = value
    elif key in ['oss version', 'version']:
        oss.version = value
    elif key in ['download location', 'source']:
        oss.download_location = value
    elif key in ['license', 'license text']:
        oss.license = value
    elif key in ['file name or path', 'source name or path', 'file', 'binary name']:
        oss.source_name_or_path = value
    elif key in ['copyright text', 'copyright']:
        oss.copyright = value
    elif key == 'exclude':
        oss.exclude = value
    elif key == 'comment':
        oss.comment = value
    elif key == 'homepage':
        oss.homepage = value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import sys
import wget
import tarfile
import zipfile
import logging
import getopt
import shutil
import pygit2 as git
import bz2
from datetime import datetime
from pathlib import Path
from ._get_downloadable_url import get_downloadable_url
import fosslight_util.constant as constant
from fosslight_util.set_log import init_log
import signal

logger = logging.getLogger(constant.LOGGER_NAME)
compression_extension = {".tar.bz2", ".tar.gz", ".tar.xz", ".tgz", ".tar", ".zip", ".jar", ".bz2"}
SIGNAL_TIMEOUT = 600


class TimeOutException(Exception):
    pass


def alarm_handler(signum, frame):
    logger.warning("git clone timeout! (%d sec)", SIGNAL_TIMEOUT)
    raise TimeOutException()


def print_help_msg():
    print("* Required : -s link_to_download")
    print("* Optional : -t target_directory")
    print("* Optional : -d log_file_directory")
    sys.exit()


def main():

    src_link = ""
    target_dir = os.getcwd()
    log_dir = os.getcwd()

    try:
        argv = sys.argv[1:]
        opts, args = getopt.getopt(argv, 'hs:t:d:')
    except getopt.GetoptError:
        print_help_msg()

    for opt, arg in opts:
        if opt == "-h":
            print_help_msg()
        elif opt == "-s":
            src_link = arg
        elif opt == "-t":
            target_dir = arg
        elif opt == "-d":
            log_dir = arg

    if src_link == "":
        print_help_msg()
    else:
        cli_download_and_extract(src_link, target_dir, log_dir)


def cli_download_and_extract(link, target_dir, log_dir, checkout_to="", compressed_only=False):
    global logger

    success = True
    msg = ""
    log_file_name = "fosslight_download_" + \
        datetime.now().strftime('%Y%m%d_%H-%M-%S')+".txt"
    logger, log_item = init_log(os.path.join(log_dir, log_file_name))

    try:
        if link == "":
            success = False
            msg = "Need a link to download."
        elif os.path.isfile(target_dir):
            success = False
            msg = "The target directory exists as a file.:"+target_dir
        else:
            if not download_git_clone(link, target_dir, checkout_to):
                if os.path.isfile(target_dir):
                    shutil.rmtree(target_dir)

                success, downloaded_file = download_wget(link, target_dir, compressed_only)
                if success:
                    success = extract_compressed_file(downloaded_file, target_dir, True)
    except Exception as error:
        success = False
        msg = str(error)

    logger.info("* FOSSLight Downloader - Result :"+str(success)+"\n"+msg)
    return success, msg


def get_ref_to_checkout(checkout_to, ref_list):
    ref_to_checkout = checkout_to
    try:
        checkout_to = checkout_to.strip()
        if checkout_to in ref_list:
            return checkout_to

        prefix_refs = ["refs/remotes/origin/", "refs/tags/"]
        for prefix in prefix_refs:
            ref_to_checkout = prefix+checkout_to
            if ref_to_checkout in ref_list:
                return ref_to_checkout

        ref_to_checkout = next(
            x for x in ref_list if x.endswith(checkout_to))
    except Exception as error:
        logger.warning("git find ref - failed:"+str(error))
    return ref_to_checkout


def download_git_clone(git_url, target_dir, checkout_to=""):
    signal.signal(signal.SIGALRM, alarm_handler)
    signal.alarm(SIGNAL_TIMEOUT)
    try:
        Path(target_dir).mkdir(parents=True, exist_ok=True)
        repo = git.clone_repository(git_url, target_dir,
                                    bare=False, repository=None,
                                    remote=None, callbacks=None)
        signal.alarm(0)
    except Exception as error:
        logger.warning("git clone - failed:"+str(error))
        return False
    try:
        ref_to_checkout = checkout_to
        if checkout_to != "":
            ref_list = [x for x in repo.references]
            ref_to_checkout = get_ref_to_checkout(checkout_to, ref_list)
            logger.info("git checkout :"+ref_to_checkout)
            repo.checkout(ref_to_checkout)
    except Exception as error:
        logger.warning("git checkout to "+ref_to_checkout +
                       " - failed:"+str(error))
    return True


def download_wget(link, target_dir, compressed_only):
    success = False
    downloaded_file = ""

    signal.signal(signal.SIGALRM, alarm_handler)
    signal.alarm(SIGNAL_TIMEOUT)
    try:
        Path(target_dir).mkdir(parents=True, exist_ok=True)

        ret, new_link = get_downloadable_url(link)
        if ret and new_link != "":
            link = new_link

        if compressed_only:
            for ext in compression_extension:
                if link.endswith(ext):
                    success = True
                    break
        else:
            success = True

        if not success:
            raise Exception('Not supported compression type (link:{0})'.format(link))

        logger.info("wget:"+link)
        downloaded_file = wget.download(link)
        signal.alarm(0)

        shutil.move(downloaded_file, target_dir)
        downloaded_file = os.path.join(target_dir, downloaded_file)
        if downloaded_file != "":
            success = True
            logger.debug("wget - downloaded:"+downloaded_file)
    except Exception as error:
        success = False
        logger.warning("wget - failed:"+str(error))

    return success, downloaded_file


def extract_compressed_dir(src_dir, target_dir, remove_after_extract=True):
    logger.debug("Extract Dir:"+src_dir)
    try:
        files_path = [os.path.join(src_dir, x) for x in os.listdir(src_dir)]
        for fname in files_path:
            extract_compressed_file(fname, target_dir, remove_after_extract)
    except Exception as error:
        logger.debug("Extract files in dir - failed:"+str(error))
        return False
    return True


def extract_compressed_file(fname, extract_path, remove_after_extract=True):
    try:
        is_compressed_file = True
        if os.path.isfile(fname):
            if fname.endswith(".tar.bz2"):
                decompress_bz2(fname, extract_path)
                os.remove(fname)
                fname = os.path.splitext(fname)[0]

            if fname.endswith(".tar.gz") or fname.endswith(".tgz"):
                tar = tarfile.open(fname, "r:gz")
                tar.extractall(path=extract_path)
                tar.close()
            elif fname.endswith(".tar.xz") or fname.endswith(".tar"):
                tar = tarfile.open(fname, "r:*")
                tar.extractall(path=extract_path)
                tar.close()
            elif fname.endswith(".zip") or fname.endswith(".jar"):
                unzip(fname, extract_path)
            elif fname.endswith(".bz2"):
                decompress_bz2(fname, extract_path)
            else:
                is_compressed_file = False
                logger.warning("Unsupported file extension:"+fname)

            if remove_after_extract and is_compressed_file:
                logger.debug("Remove - extracted file :"+fname)
                os.remove(fname)
        else:
            logger.warning("Not a file:"+fname)
    except Exception as error:
        logger.error("Extract - failed:"+str(error))
        return False
    return True


def decompress_bz2(source_file, dest_path):
    try:
        fzip = bz2.BZ2File(source_file)
        data = fzip.read()  # get the decompressed data
        open(os.path.splitext(source_file)[0], 'wb').write(data)  # write a uncompressed file

    except Exception as error:
        logger.error("Decompress bz2 - failed:"+str(error))
        return False
    return True


def unzip(source_file, dest_path):
    try:
        fzip = zipfile.ZipFile(source_file, 'r')
        for filename in fzip.namelist():
            fzip.extract(filename, dest_path)
        fzip.close()
    except Exception as error:
        logger.error("Unzip - failed:"+str(error))
        return False
    return True


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import re
from bs4 import BeautifulSoup
from urllib.request import urlopen
import fosslight_util.constant as constant

logger = logging.getLogger(constant.LOGGER_NAME)


def get_downloadable_url(link):

    ret = False
    new_link = ''

    link = link.replace('http://', '')
    link = link.replace('https://', '')

    if link.startswith('pypi.org/'):
        ret, new_link = get_download_location_for_pypi(link)
    elif link.startswith('mvnrepository.com/artifact/') or link.startswith('repo1.maven.org/'):
        ret, new_link = get_download_location_for_maven(link)
    elif link.startswith('www.npmjs.com/') or link.startswith('registry.npmjs.org'):
        ret, new_link = get_download_location_for_npm(link)
    elif link.startswith('pub.dev/'):
        ret, new_link = get_download_location_for_pub(link)

    return ret, new_link


def get_download_location_for_pypi(link):
    # get the url for downloading source file in pypi.org/project/(oss_name)/(oss_version)/#files
    ret = False
    new_link = ''

    try:
        dn_loc_re = re.findall(r'pypi.org\/project\/?([^\/]*)\/?([^\/]*)', link)
        oss_name = dn_loc_re[0][0]
        oss_version = dn_loc_re[0][1]

        pypi_url = 'https://pypi.org/project/' + oss_name + '/' + oss_version + '/#files'

        content = urlopen(pypi_url).read().decode('utf8')
        bs_obj = BeautifulSoup(content, 'html.parser')

        card_file_list = bs_obj.findAll('div', {'class': 'card file__card'})

        for card_file in card_file_list:
            file_code = card_file.find('code').text
            if file_code == "source":
                new_link = card_file.find('a').attrs['href']
                ret = True
                break
    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for pypi (url:'+link+') '+str(error))

    return ret, new_link


def get_download_location_for_maven(link):
    # get the url for downloading source file in
    # repo1.maven.org/maven2/(group_id(split to separator '/'))/(artifact_id)/(oss_version)
    ret = False
    new_link = ''

    try:
        if link.startswith('mvnrepository.com/artifact/'):
            dn_loc_split = link.replace('mvnrepository.com/', '').split('/')
            group_id = dn_loc_split[1].replace('.', '/')
            dn_loc = 'https://repo1.maven.org/maven2/' + group_id + '/' + dn_loc_split[2] + '/' + dn_loc_split[3]

        elif link.startswith('repo1.maven.org/maven2/'):
            dn_loc_split = link.replace('repo1.maven.org/maven2/', '').split('/')

            if link.endswith('.tar.gz') or link.endswith('.jar') or link.endswith('.tar.xz'):
                new_link = 'https://' + link
                ret = True
                return ret, new_link
            else:
                dn_loc = 'https://' + link
        else:
            raise Exception("not valid url for maven")

        html = urlopen(dn_loc).read().decode('utf8')
        bs_obj = BeautifulSoup(html, 'html.parser')

        file_name = dn_loc.split('/')[-2] + '-' + dn_loc.split('/')[-1] + '-sources.jar'

        for link in bs_obj.findAll("a"):
            if link.text == file_name:
                source_url = link['href']
                new_link = dn_loc + '/' + source_url
                break
            elif link['href'].endswith('sources.jar') or link['href'].endswith('source.jar') or link['href'].endswith('src.jar'):
                source_url = link['href']
                new_link = dn_loc + '/' + source_url

        if new_link != '':
            ret = True

    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for maven (url:'+link+') '+str(error))

    return ret, new_link


def get_download_location_for_npm(link):
    # url format : registry.npmjs.org/packagename/-/packagename-version.tgz
    ret = False
    new_link = ''

    try:
        if link.startswith('www.npmjs.com/') or link.startswith('registry.npmjs.org'):
            dn_loc_split = link.split('/')

            if dn_loc_split[1] == 'package':
                idx = 2
            else:
                idx = 1

            if dn_loc_split[idx].startswith('@'):
                oss_name_npm = dn_loc_split[idx]+'/'+dn_loc_split[idx+1]
                tar_name = dn_loc_split[idx+1] + '-' + dn_loc_split[idx+3]
            else:
                oss_name_npm = dn_loc_split[idx]
                tar_name = oss_name_npm + '-' + dn_loc_split[idx+2]

            new_link = 'https://registry.npmjs.org/' + oss_name_npm + '/-/' + tar_name + '.tgz'
            ret = True

    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for npm (url:'+link+') '+str(error))

    return ret, new_link


def get_download_location_for_pub(link):
    ret = False
    new_link = ''

    # url format : https://pub.dev/packages/(oss_name)/versions/(oss_version)
    # download url format : https://storage.googleapis.com/pub-packages/packages/(oss_name)-(oss_version).tar.gz
    try:
        if link.startswith('pub.dev/packages'):
            dn_loc_split = link.split('/')
            oss_name_pub = dn_loc_split[2]
            oss_version_pub = dn_loc_split[4]

            new_link = 'https://storage.googleapis.com/pub-packages/packages/' + oss_name_pub + '-' + oss_version_pub + '.tar.gz'
            ret = True

    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for npm (url:'+link+') '+str(error))

    return ret, new_link
